$title Combine IEA97 and GTAP5 data

$offlisting
$batinclude ..\inclib\mrtdata iea
$onlisting

$label start

alias (r,s);
alias (i,j);

*	Identify energy sectors:

set e(i) /
GAS	Natural gas works
ELY	Electricity and heat 
OIL	Refined oil products
COL	Coal transformation
P_C	Petroleum and coal products 
* OEN	Other energy 
/;

*	Read energy price and quantity data:

$offlisting
$include ..\data\energy.dat
$onlisting

set usr Users for which prices are differentiated /
	I	Industry
	H	Household
	U	Utility
	X	Export  /;
 
set elec(i)/ely/, cru(i)/oil/, oil(i)/p_c/, gas(i)/gas/, col(i)/col/;
set nele(i) /p_c, gas, col/;

parameter	etrade	Energy trade
		eprice	International energy prices (implicit);

etrade(e,r,"export") = sum(s, vxmd(e,r,s));
etrade(e,r,"import") = sum(s, vxmd(e,s,r));
etrade(e,r,"txbar")$sum(s, vxmd(e,r,s))
 = 100 * sum(s, vxmd(e,r,s)*tx(e,r,s))/sum(s, vxmd(e,r,s));

etrade(e,r,"tmbar")$sum(s, (1+tx(e,s,r))*vxmd(e,s,r))
 = 100 * sum(s, tm(e,s,r)*((1+tx(e,s,r)) * vxmd(e,s,r))) /
			sum(s, (1+tx(e,s,r))*vxmd(e,s,r));

etrade(e,r,"tmbar_tr")$sum(s, (1+tx(e,s,r))*vxmd(e,s,r))
 = 100 * sum(s, tm(e,s,r)*((1+tx(e,s,r)) * vxmd(e,s,r))+vtwr(e,s,r)) /
			sum(s, (1+tx(e,s,r))*vxmd(e,s,r)+vtwr(e,s,r));

etrade(e,r,"tmbar_my")$sum(s, vxmd(e,s,r))
 = 100 * sum(s, tm(e,s,r)* vxmd(e,s,r)) /
			sum(s, vxmd(e,s,r));

etrade(e,r,"export-xj") = eexp(e,r);
etrade(e,r,"import-xj") = eimp(e,r);

eprice(e,r,"export")$eexp(e,r) = etrade(e,r,"export")/eexp(e,r);
eprice(e,r,"import")$eimp(e,r) = etrade(e,r,"import")/eimp(e,r);
eprice(e,"average","export")$sum(r,eexp(e,r)) = sum(r, etrade(e,r,"export"))/sum(r,eexp(e,r));
eprice(e,"average","import")$sum(r,eimp(e,r)) = sum(r, etrade(e,r,"import"))/sum(r,eimp(e,r));

option etrade:1:1:1, eprice:1:1:1;
display etrade, eprice;

*	-----------------------------------------------------------
*	Move the dataset into a zip archive:

$call 'copy ..\defines\iea.set ..\defines\gtapiea.set >nul'
$call 'zip -j ..\data\gtapiea ..\defines\gtapiea.set >nul'
$call 'zip -j ..\data\gtapiea ..\defines\iea.set >nul';
$call 'zip -j ..\data\gtapiea ..\defines\iea.map >nul';
$batinclude ..\inclib\zip ..\data\ gtapiea ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm vdpm vipm vkb vdep vsave eind efd eexp eimp 
