$title  filter.gms: GAMS code which filters and then recalibrates the GTAP data

*       -----------------------------------------------------------
*       Read a filter tolerance from an external file?

$setglobal version   4
$setglobal tolerance 001
$setglobal source gtap
$setglobal target gtap001

$if exist filter.inc $include filter.inc

$if not exist ..\defines\%target%.set $call 'copy ..\defines\%source%.set ..\defines\%target%.set'

scalar  tolerance  Relative tolerance for filtering trade and production data /0.%tolerance%/;

*       Begin by reading the GTAP set definitions:

$include ..\defines\%source%.set

alias (i,j,k), (r,s,ss);

SET     cgd(i) /cgd/;

SET     D Classes of demand for which import shares are differentiated / 
        G       Public, 
        C       Private Consumption,
        I       Intermediate input /;

*       -----------------------------------------------------------
*       Read data from the archive:

parameter
        ty(i,r)         Output tax
        ti(j,i,r)       Intermediate input tax
        tf(f,i,r)       Factor tax
        tx(i,s,r)       Export tax rate (defined on a net basis)
        tm(i,s,r)       Import tariff rate
        tg(i,r)         Tax rates on government demand
        tp(i,r)         Tax rate on private demand

        vafm(j,i,r)     Aggregate intermediate inputs
        vfm(f,i,r)      Value of factor inputs (net of tax)
        vxmd(i,r,s)     Value of commodity trade (fob - net export tax)
        vtwr(i,r,s)     Transport services
        vst(i,r)        Value of international transport sales
        vdgm(i,r)       Government demand (domestic)
        vigm(i,r)       Government demand (imported)
        vdpm(i,r)       Private demands (domestic)
        vipm(i,r)       Private demands (imported)

	vsave(r)	Regional savings
	vkb(r)		Value of capital stock (beginning of period)
	vdep(r)		Value of capital depreciation;


$batinclude ..\inclib\unzip %source% gms

*       -----------------------------------------------------------
*       Compute intermediate values:

parameter
        vim(i,r)        Total value of imports (gross tariff)
        vxm(i,r)        Value of export (net excise tax)
        vdm(i,r)        Value of domestic output (net excise tax)
        vdfm(i,r)       Aggregate intermediate demand (domestic)
        vifm(i,r)       Aggregate intermediate demand (imported)
        market(i,r)     Market clearance condition
        vpm(i,r)        Aggregate private demand
        vgm(i,r)        Aggregate public demand;


*       Infer exports and imports from the trade matrix, 
*       domestic production from exhaustion of product:

vxm(i,r) = sum(s, vxmd(i,r,s)) + vst(i,r) ;
vim(i,r) = sum(s, (vxmd(i,s,r) * (1+tx(i,s,r)) + vtwr(i,s,r) ) * (1+tm(i,s,r)) );
vdm(i,r) = (sum(j, vafm(j,i,r)*(1+ti(j,i,r))) + sum(f, vfm(f,i,r)*(1+tf(f,i,r))))/(1-ty(i,r)) - vxm(i,r);
vdfm(i,r) = vdm(i,r) - vdgm(i,r) - vdpm(i,r) - vdm(i,r)$cgd(i);
vifm(i,r) = vim(i,r) - vipm(i,r) - vigm(i,r);

*       -----------------------------------------------------------
*       Check the original equilibrium:

$include chkeq

marketi(r,i) = round( vdfm(i,r) + vifm(i,r) - sum(j, vafm(i,j,r)), ndchk);

*       -----------------------------------------------------------
*       Round all tax rates to the nearest percentage:

ty(i,r) = round(ty(i,r), 2);
ti(j,i,r) = round(ti(j,i,r),2);
tf(f,i,r) = round(tf(f,i,r),2);
tx(i,s,r) = round(tx(i,s,r),2);
tm(i,s,r) = round(tm(i,s,r),2);
tg(i,r) = round(tg(i,r),2);
tp(i,r) = round(tp(i,r),2);

*       -----------------------------------------------------------
*       Round all entries to the closest $100,000:

vafm(j,i,r) = round( vafm(j,i,r), 6);
vfm(f,i,r) = round( vfm(f,i,r), 6);
vxmd(i,r,s) = round( vxmd(i,r,s), 6);
vtwr(i,r,s) = round( vtwr(i,r,s), 6);
vst(i,r)  = round( vst(i,r), 6);
vdgm(i,r) = round( vdgm(i,r), 6);
vigm(i,r) = round( vigm(i,r), 6);
vdpm(i,r) = round( vdpm(i,r), 6);
vipm(i,r) = round( vipm(i,r), 6);

*       -----------------------------------------------------------
*       Recompute intermediate values:

*       Infer exports and imports from the trade matrix, 
*       domestic production from exhaustion of product:

vxm(i,r) = sum(s, vxmd(i,r,s)) + vst(i,r);
vim(i,r) = sum(s, (vxmd(i,s,r) * (1+tx(i,s,r)) + vtwr(i,s,r) ) * (1+tm(i,s,r)) );
vdm(i,r) = (sum(j, vafm(j,i,r)*(1+ti(j,i,r))) + sum(f, vfm(f,i,r)*(1+tf(f,i,r))))/(1-ty(i,r)) - vxm(i,r);
vdfm(i,r) = vdm(i,r) - vdgm(i,r) - vdpm(i,r) - vdm(i,r)$cgd(i);
vifm(i,r) = vim(i,r) - vipm(i,r) - vigm(i,r);

parameter magnitude, vy;

*       -----------------------------------------------------------
*       Filter the trade data:

*       Retain original data for rescaling:

parameter 
        vimn    VXMD row sums before filtering, 
        vtw     VTWR row sums before filtering
        yt0     Transport demand subtotals
        density summary of changes in matrix density;

vimn(i,r) = sum(s, vxmd(i,s,r));
vtw(i,r)  = sum(s, vtwr(i,s,r));

*       Begin by eliminating imports and exports which amount
*       to less than target share of aggregate demand or supply:

vim(i,r)$(vim(i,r) lt tolerance*(vim(i,r)+vdm(i,r))) = 0;
vxm(i,r)$(vxm(i,r) lt tolerance * (vxm(i,r)+vdm(i,r))) = 0;

vigm(i,r)$(  not vim(i,r)) = 0;
vipm(i,r)$(  not vim(i,r)) = 0;
vxmd(i,s,r)$((not vim(i,r)) or (not vxm(i,s))) = 0;

*       -----------------------------------------------------------
*       Define magnitudes:

magnitude(i,s,r)$vxmd(i,s,r) =  100 * max( vxmd(i,s,r)/(vxm(i,s)-vst(i,s)), 
                  (vxmd(i,s,r)*(1+tx(i,s,r))+vtwr(i,s,r))*(1+tm(i,s,r))/vim(i,r) );

density("trade","before") = 100 * sum((i,r,s)$vxmd(i,r,s), 1) / (card(i)*card(r)*card(r));

vxmd(i,r,s)$(magnitude(i,r,s) lt 100 * tolerance) = 0;

*	Following Joe Francois' suggestion, I've retained own-region trade here:

*.Move own-region export-import into domestic supply:
*.vdm(i,r) = vdm(i,r) + vxmd(i,r,r);
*.vxmd(i,r,r) = 0;

tm(i,s,r)$(not vxmd(i,s,r)) = 0;
tx(i,s,r)$(not vxmd(i,s,r)) = 0;
vtwr(i,s,r)$(not vxmd(i,s,r)) = 0;

*       Rescale flows to retain consistent values of aggregate imports,
*       tariff revenue, export tax payments and transport costs:

vxmd(i,s,r)$sum(ss, vxmd(i,ss,r)) = vxmd(i,s,r) * vimn(i,r) / sum(ss, vxmd(i,ss,r));
vtwr(i,s,r)$sum(ss, vtwr(i,ss,r)) = vtwr(i,s,r) * vtw(i,r)  / sum(ss, vtwr(i,ss,r));


density("trade","after") = 100 * sum((i,r,s)$vxmd(i,r,s), 1) / (card(i)*card(r)*card(r));

magnitude(i,r,s) = 0;

*       ----------------------------------------
*       Filter the production data:

vy(i,r) = (vdm(i,r)+vxm(i,r)) * (1-ty(i,r));

magnitude(i,j,r)$min(vy(j,r), vdm(i,r)+vim(i,r))
        = 100 * max( (vafm(i,j,r)*(1 + ti(i,j,r))  / vy(j,r))$vy(j,r), 
		     (vafm(i,j,r) / (vdm(i,r)+vim(i,r)))$(vdm(i,r)+vim(i,r)) );

magnitude(f,i,r)$vy(i,r) = 100 * vfm(f,i,r) *(1 + tf(f,i,r)) / vy(i,r);

density("prod","before") = 100 * sum((i,j,r)$vafm(i,j,r), 1) / (card(i)*card(i)*card(r));

vafm(i,j,r)$(magnitude(i,j,r)  lt 100 * tolerance) = 0;
vfm(f,i,r)$(magnitude(f,i,r)   lt 100 * tolerance) = 0;

ti(i,j,r)$(not vafm(i,j,r)) = 0;
tf(f,i,r)$(not  vfm(f,i,r)) = 0;

density("prod","after") = 100 * sum((i,j,r)$vafm(i,j,r), 1) / (card(i)*card(i)*card(r));

display density;

*       ----------------------------------------
*       Filter final demands as well:

parameter vp, vg;

vpm(i,r) = vipm(i,r) + vdpm(i,r);
vdpm(i,r)$(vdpm(i,r) le tolerance * vpm(i,r)) = 0;
vipm(i,r)$(vipm(i,r) le tolerance * vpm(i,r)) = 0;

vpm(i,r) = vipm(i,r) + vdpm(i,r);
vp(r) = sum(i, vpm(i,r)*(1+tp(i,r)));
vpm(i,r)$(vpm(i,r) le tolerance * vp(r)) = 0;

vdpm(i,r)$(not vpm(i,r)) = 0;
vipm(i,r)$(not vpm(i,r)) = 0;
tp(i,r)$(not vpm(i,r)) = 0;


vgm(i,r) = vigm(i,r) + vdgm(i,r);
vdgm(i,r)$(vdgm(i,r) le tolerance * vgm(i,r)) = 0;
vigm(i,r)$(vigm(i,r) le tolerance * vgm(i,r)) = 0;
vgm(i,r) = vigm(i,r) + vdgm(i,r);
vg(r) = sum(i, vgm(i,r)*(1+tg(i,r)));
vgm(i,r)$(vgm(i,r) le tolerance * vg(r)) = 0;
vdgm(i,r)$(not vgm(i,r)) = 0;
vigm(i,r)$(not vgm(i,r)) = 0;
tg(i,r)$(not vgm(i,r)) = 0;

vst(i,r)$(vst(i,r) le tolerance * sum(j, vst(j,r))) =  0;

*       -----------------------------------------------------------
*       Generate a report of extreme values for tax rates

parameter
        xtf(f,i,r)       Extreme factor tax
        xtp(i,r)         Extreme tax rate on private demand
        xtg(i,r)         Extreme tax rates on government demand
        xtx(i,s,r)       Extreme export tax rate (defined on a net basis)
        xtm(i,s,r)       Extreme import tariff rate
        xty(i,r)         Extreme output tax
        xti(j,i,r)       Extreme intermediate input tax;

*       TY is on a gross basis, so display taxes in excess of 50%:

XTY(I,R)$((TY(I,R) LE -.50) or (TY(I,R) ge .50)) = 100 * TY(I,R);

*       All other taxes are on a net basis so display taxes in excess of 200%:

XTI(J,I,R)$((TI(J,I,R) LE -.50) or (TI(J,I,R) GE 4)) = 100 * TI(J,I,R);
XTF(F,I,R)$((TF(F,I,R) LE -.50) or (TF(F,I,R) GE 2)) = 100 * TF(F,I,R);
XTX(I,R,S)$((TX(I,S,R) LE -.50) or (TX(I,S,R) GE 2)) = 100 * TX(I,S,R);
XTM(I,S,R)$((TM(I,S,R) LE -.50) or (TM(I,S,R) GE 2)) = 100 * TM(I,S,R);
XTG(I,R)$((TG(I,R) LE -.50) or (TG(I,R) GE 2)) = 100 * TG(I,R);
XTP(I,R)$((TP(I,R) LE -.50) or (TP(I,R) GE 2)) = 100 * TP(I,R);

option xtm:0:1:1, xtx:0:1:1, xti:0:1:1, XTY:0;
display  xti, xtf, xtx, xtm, xty, xtg, xtp;

*       Place an upper bound of 1000% on intermediate input taxes:

TI(J,I,R) = MIN(TI(J,I,R), 10);

*       -----------------------------------------------------------
*       Recalibrate the dataset:

$include recalib

$if setglobal skipcrude $goto filtervafm

*       -----------------------------------------------------------
*       Move all crude oil inputs to the refining sector:

set     oil(i)  refined oil /p_c/
        cru(i)  crude oil   /oil/;

*       Revise crude oil demand:

parameter   crudeinput  crude inputs to non-refining activities;

vpm(i,r) = vipm(i,r) + vdpm(i,r);
vgm(i,r) = vigm(i,r) + vdgm(i,r);

loop(cru,

  crudeinput(r,i) = vafm(cru,i,r)$(not oil(i));
  crudeinput(r,"fd") = vpm(cru,r) + vgm(cru,r);

  ti(oil,i,r)$(vafm(oil,i,r) + crudeinput(r,i))
        = (ti(oil,i,r)*vafm(oil,i,r)+ti(cru,i,r)*crudeinput(r,i))
                                / (vafm(oil,i,r) + crudeinput(r,i));
  vafm(oil,i,r) = vafm(oil,i,r) + crudeinput(r,i);
  vafm(cru,i,r) = vafm(cru,i,r) - crudeinput(r,i);

  tp(oil,r)$(vpm(oil,r)+vpm(cru,r))
        = (tp(oil,r)*vpm(oil,r)+tp(cru,r)*vpm(cru,r))/(vpm(oil,r)+vpm(cru,r));
  vpm(oil,r) = vpm(oil,r) + vpm(cru,r);
  vdpm(oil,r) = vdpm(oil,r) + vpm(cru,r);

  vpm(cru,r) = 0;
  vdpm(cru,r) = 0;
  vipm(cru,r) = 0;

  tg(oil,r)$(vgm(oil,r)+vgm(cru,r))
        = (tg(oil,r)*vgm(oil,r)+tg(cru,r)*vgm(cru,r))/(vgm(oil,r)+vgm(cru,r));
  vgm(oil,r) = vgm(oil,r) + vgm(cru,r);
  vdgm(oil,r) = vdgm(oil,r) + vgm(cru,r);

  vgm(cru,r) = 0;
  vdgm(cru,r) = 0;
  vigm(cru,r) = 0;

  ty(oil,r)$(vdm(oil,r)+vxm(oil,r)+sum(i, crudeinput(r,i)) + crudeinput(r,"fd"))
    = ty(oil,r) * (vdm(oil,r)+vxm(oil,r))
        / (vdm(oil,r)+vxm(oil,r)+sum(i, crudeinput(r,i)) + crudeinput(r,"fd"));

  vdm(oil,r) = vdm(oil,r) + sum(i, crudeinput(r,i)) + crudeinput(r,"fd");
  
  ti(cru,oil,r)$(vafm(cru,oil,r)+sum(i,crudeinput(r,i))+crudeinput(r,"fd"))
    = ti(cru,oil,r)*vafm(cru,oil,r) /
          (vafm(cru,oil,r)+sum(i,crudeinput(r,i))+crudeinput(r,"fd"));

  vafm(cru,oil,r) = vafm(cru,oil,r) + sum(i,crudeinput(r,i)) + crudeinput(r,"fd");

);

$label filtervafm

*	Avoid tedious negative numbers:

vafm(i,j,r) = max(0, vafm(i,j,r));
vdfm(i,r) = max(0, vdfm(i,r));
vifm(i,r) = max(0, vifm(i,r));


*       Move the dataset into a zip archive:

$batinclude ..\inclib\zip ..\data\ %target% ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm vdpm vipm vsave vdep vkb
execute 'zip -j ..\data\%target% ..\defines\%target%.set';


*       Generate a GDP report:

parameter
        pmx0(i,s,r)     Reference price of imports,
        pmt0(i,s,r)     Reference price of transport services,
        pg0(i,r)        Reference price of government demand,
        pc0(i,r)        Reference price for private demand,
        py0(i,r)        Reference output price (domestic or export),
        pai0(j,i,r)     Reference price for intermediate inputs,
        pf0(f,i,r)      Reference price for factor inputs;

pg0(i,r) = 1 + tg(i,r);
pc0(i,r) = 1 + tp(i,r);
pmt0(i,s,r) = (1 + tm(i,s,r));
pmx0(i,s,r) = (1 + tm(i,s,r)) * (1 + tx(i,s,r));
py0(i,r) =1 - ty(i,r);
pf0(f,j,s) = 1 + tf(f,j,s);

$include ..\inclib\gdpreport
